@@ -62,7 +62,7 @@ class Sf:
     def authenticate(self, sfUsername: Optional[str]=None, 
                                 sfPassword: Optional[str]=None, 
                                 sfOrgId: Optional[str]=None, 
-                                isSandbox: Optional[str]=None, 
+                                isSandbox: Optional[bool]=None, 
                                 session_id: Optional[str]=None, 
                                 instance: Optional[str]=None, 
                                 security_token: Optional[str] = None,
@@ -93,8 +93,10 @@ class Sf:
         # If session ID already exists and instance URL is already populated,
         # reauthenticate using existing session ID
         if session_id is not None and instance is not None:
-            sf = Salesforce(session_id = session_id, instance_url = instance)
+            sf = Salesforce(session_id = session_id, instance = instance)
             self.sf = sf
+            self.instance = instance
+            self.session_id = session_id
         
         # If username, password, org ID, and isSandbox flags are all provided,
         # authenticate using provided credentials
@@ -142,10 +144,10 @@ class Sf:
 
         # Alternative way to authenticate using SFDC Bulk API
         # bulk = SalesforceBulk(username=sfUsername, password=sfPassword, security_token='')
-        bulk = SalesforceBulk(sessionId = session_id, host = instance)
+        bulk = SalesforceBulk(sessionId = self.session_id, host = self.instance)
         self.bulk = bulk
         # SFDC Metadata API
-        sfMeta = SfdcSession(session_id=session_id, instance=".".join(instance.split(".")[:2]))
+        sfMeta = SfdcSession(session_id=self.session_id, instance=".".join(self.instance.split(".")[:2]))
         self.sfMeta = sfMeta
         # Alternative way to authenticate using SFDC Metadata API
         # sfMeta = SfdcSession(username=sfUsername,password=sfPassword,token='',is_sandbox=isSandbox)
@@ -173,7 +175,7 @@ class Sf:
     ### Synchronous Data Functions
     ### ----------------------------------------------------------------------------------------------------
 
-    def query(self, query: str, excludedFields: Optional[List] = []) -> pd.DataFrame:
+    def query(self, query: str, excludedFields: Optional[List] = [], *args) -> pd.DataFrame:
         """
         Using SFDC SOQL Syntax, and allowing for Relationships and group bys. 
         
@@ -352,6 +354,7 @@ class Sf:
             result = unpack_column(result, "errors")
 
         return pd.DataFrame(result.rename(columns={'id': 'Id'}))
+    
     def update(self, object_api: str=None, record_dataframe: pd.DataFrame = pd.DataFrame()) -> pd.DataFrame:
         """
         Function updates records within the Salesforce instance using bulk api.
@@ -544,6 +547,10 @@ class Sf:
     ### Asynchronous Data Functions
     ### ----------------------------------------------------------------------------------------------------
 
+    async def async_query(self, query, excludedFields: Optional[List]= []) -> pd.DataFrame:
+        async_query = async_wrap(self.query)
+        return await async_query(query, excludedFields)
+
     async def async_upsert(self, object_api, record_dataframe, external_id_field_api, batchsize=2000, *args, **kwargs):
         async_upsert = async_wrap(self.upsert)
         batches = len(record_dataframe) / batchsize
