@@ -27,6 +27,7 @@ class Vn:
         self.networkReferenceTypes: dict = None # {'AddressAdminArea': {'type': 'AddressAdminArea','customerOwned': False,'inactive': False,'description': 'AddressAdminArea'}, 'AddressCBSA': {'type': 'AddressCBSA' ... }}
         self.networkReferenceValueMetadata: dict = None # Dictionary of Network Reference Value Metadata, including countries, reference codes, translated values, etc.
         self.sessionId: str = None
+        self.networkId: str = None
         self.APIheaders: dict = None
         self.APIversionList: list = []
         self.LatestAPIversion: str = None
@@ -171,6 +172,8 @@ class Vn:
         networkUserName: str=None, 
         networkPassword: str=None, 
         networkCountry: str=None, 
+        networkId: str=None,
+        sessionId: str=None,
         if_return: bool=False, 
         *args, **kwargs
     ) -> Optional[dict]:
@@ -198,7 +201,9 @@ class Vn:
         https://developer.veevanetwork.com/API_reference/API_reference.htm#Authenticate
         
         """
-        
+        if (networkId and sessionId and networkURL):
+            self.networkId = self.networkId if networkId is None else networkId
+            self.sessionId = self.sessionId if sessionId is None else sessionId
         
         url_parse = urlparse(networkURL)
         if len(url_parse.scheme) == 0:
@@ -216,23 +221,32 @@ class Vn:
         if (self.networkURL is None) or (len(self.networkURL) == 0):
             raise Exception('networkURL is required')
 
-            
 
         # self.networkURL = self.networkURL if networkURL is None else networkURL
         self.networkUserName = self.networkUserName if networkUserName is None else networkUserName
         self.networkPassword = self.networkPassword if networkPassword is None else networkPassword
         self.networkCountry = self.networkCountry if networkCountry is None else networkCountry
         
+        if (self.networkUserName and self.networkPassword):
+            self.networkConnection = requests.post(self.networkURL + '/api/v17.0/auth?username=' + self.networkUserName + '&password=' + self.networkPassword)
+            self.sessionId = self.networkConnection.json()['sessionId']
+            self.userId = self.networkConnection.json()['userId']
+            self.networkId = self.networkConnection.json()['networkId']
         
-        self.networkConnection = requests.post(self.networkURL + '/api/v17.0/auth?username=' + self.networkUserName + '&password=' + self.networkPassword)
-        self.sessionId = self.networkConnection.json()['sessionId']
         self.APIheaders = {'authorization': self.sessionId}
         self.APIversionList = []
+        
+        # Error checking whether the required parameters are passed in
+        # The check happens here because this is where all the self assignments has completed
+        if (not (self.networkId and self.sessionId and self.networkURL)) and (not (self.networkUserName and self.networkPassword and self.networkCountry and self.networkURL)):
+            raise Exception("Please provide either networkId, sessionId, and networkURL or networkUserName, networkPassword, and networkURL")
+        
         for API in requests.get(self.networkURL +'/api', headers=self.APIheaders).json()['values'].keys():
             self.APIversionList.append(float(API.replace("v", "")))
         self.APIversionList.sort()
         self.LatestAPIversion = "v" + str(self.APIversionList[-1])
         
+
         if if_return:
             return {'networkURL':self.networkURL, 
                     'networkUserName':self.networkUserName, 
