@@ -2,6 +2,8 @@ from sys import platform
 import requests
 import pandas as pd
 import os
+import json
+from urllib.parse import urlparse
 
 class Vv:
     def __init__(self):
@@ -10,6 +12,8 @@ class Vv:
         self.vaultPassword = None
         self.vaultConnection = None
         self.sessionId = None
+        self.vaultId: str = None
+        self.vaultDNS: str = None
         self.APIheaders = None
         self.APIversionList = []
         self.LatestAPIversion = 'v21.3'
@@ -20,26 +24,54 @@ class Vv:
 
 
 
-    def authenticate_vv(self, vaultURL=None, vaultUserName=None, vaultPassword=None, if_return=False, *args, **kwargs):
+    def authenticate(self, 
+                     vaultURL=None, 
+                     vaultUserName=None, 
+                     vaultPassword=None, 
+                     sessionId=None,
+                     vaultId=None,
+                     if_return=False, *args, **kwargs):
         """
         TODO: Docs
         """
 
         
-        vaultURL = self.vaultURL if vaultURL is None else vaultURL
-        vaultUserName = self.vaultUserName if vaultUserName is None else vaultUserName
-        vaultPassword = self.vaultPassword if vaultPassword is None else vaultPassword
+        self.vaultURL = self.vaultURL if vaultURL is None else vaultURL
+        self.vaultUserName = self.vaultUserName if vaultUserName is None else vaultUserName
+        self.vaultPassword = self.vaultPassword if vaultPassword is None else vaultPassword
+        self.sessionId = self.sessionId if sessionId is None else sessionId
+        self.vaultId = self.vaultId if vaultId is None else vaultId
         
-        self.vaultUserName = vaultUserName
-        self.vaultURL = vaultURL
-        self.vaultPassword = vaultPassword
+        url_parse = urlparse(self.vaultURL)
+        if len(url_parse.scheme) == 0:
+            self.network_protocol = 'https'
+            if len(url_parse.path) > 0:
+                self.vaultDNS = url_parse.path
+                self.vaultURL = self.network_protocol + '://' + url_parse.path
+
+        if len(url_parse.scheme) > 0:
+            self.network_protocol = url_parse.scheme
+            if len(url_parse.netloc) > 0:
+                self.vaultDNS = url_parse.netloc
+                self.vaultURL = url_parse.scheme + '://' + url_parse.netloc
+
+        if (self.vaultURL is None) or (len(self.vaultURL) == 0):
+            raise Exception('vaultURL is required')
         
-        pload = {'username': vaultUserName,'password': vaultPassword}
-        self.vaultConnection = requests.post(f'https://{vaultURL}/api/{self.LatestAPIversion}/auth',data = pload)
-        self.sessionId = self.vaultConnection.json()['sessionId']
+        if (self.vaultUserName and self.vaultPassword and self.vaultURL):
+            pload = {'username': self.vaultUserName,'password': self.vaultPassword}
+            self.vaultConnection = requests.post(f'{self.vaultURL}/api/{self.LatestAPIversion}/auth',data = pload)
+            self.sessionId = self.vaultConnection.json()['sessionId']
+            
         self.APIheaders = {'Authorization': self.sessionId}
         self.APIversionList = []
-        for API in requests.get('https://' + vaultURL +'/api', headers=self.APIheaders).json()['values'].keys():
+        
+        # Error checking whether the required parameters are passed in
+        # The check happens here because this is where all the self assignments has completed
+        if (not (self.vaultId and self.sessionId and self.vaultURL)) and (not (self.vaultUserName and self.vaultPassword and  self.vaultURL)):
+            raise Exception("Please provide either vaultId, sessionId and vaultURL or vaultUserName, vaultPassword and vaultURL")
+        
+        for API in requests.get(self.vaultURL +'/api', headers=self.APIheaders).json()['values'].keys():
             self.APIversionList.append(float(API.replace("v", "")))
         self.APIversionList.sort()
         self.LatestAPIversion = "v" + str(self.APIversionList[-1])
@@ -53,8 +85,10 @@ class Vv:
                     'APIheaders':self.APIheaders, 
                     'APIversionList':self.APIversionList, 
                     'LatestAPIversion':self.LatestAPIversion}
+            
+            
     def query(self, query):
-        url = f"https://{self.vaultURL}/api/{self.LatestAPIversion}/query"
+        url = f"{self.vaultURL}/api/{self.LatestAPIversion}/query"
         
         h = {
         "X-VaultAPI-DescribeQuery":"true",
@@ -71,7 +105,7 @@ class Vv:
         return r
     
     def bulk_query(self, query):
-        url = f"https://{self.vaultURL}/api/{self.LatestAPIversion}/query"
+        url = f"{self.vaultURL}/api/{self.LatestAPIversion}/query"
         
         h = {
         "X-VaultAPI-DescribeQuery":"true",
@@ -93,7 +127,7 @@ class Vv:
             page_count = 1000
             
             while more_pages:
-                r = pd.DataFrame(requests.get(f"https://{self.vaultURL}"+ next_page_url+ str(page_count), headers=h).json()['data'])
+                r = pd.DataFrame(requests.get(f"{self.vaultURL}"+ next_page_url+ str(page_count), headers=h).json()['data'])
                 if len(r) == 0:
                     more_pages = False
                 else:
@@ -105,12 +139,12 @@ class Vv:
         return output
     
     def object_field_metadata(self, object_api_name):
-        url = f"https://{self.vaultURL}/api/{self.LatestAPIversion}/metadata/vobjects/{object_api_name}"
+        url = f"{self.vaultURL}/api/{self.LatestAPIversion}/metadata/vobjects/{object_api_name}"
         r = requests.get(url, headers = self.APIheaders).json()['object']['fields']
         return pd.DataFrame(r)
     
     def describe_objects(self):
-        url = f"https://{self.vaultURL}/api/{self.LatestAPIversion}/metadata/vobjects"
+        url = f"{self.vaultURL}/api/{self.LatestAPIversion}/metadata/vobjects"
         r = requests.get(url, headers = self.APIheaders).json()['objects']
         return pd.DataFrame(r).sort_values(by='name')
         
\ No newline at end of file
